@@ -6,14 +6,24 @@ use warnings;
 use DynaLoader ();
 
 use vars qw( $VERSION @ISA @EXPORT @EXPORT_OK );
-$VERSION   = "0.29";
+$VERSION   = "0.36";
 @ISA       = qw( DynaLoader Exporter );
-@EXPORT    = qw( DDumper DDsort DPeek DDisplay DDump DDual DGrow );
-@EXPORT_OK = qw( triplevar );
-$] >= 5.007003 and push @EXPORT, "DDump_IO";
+@EXPORT    = qw( DDumper DTidy DDsort DPeek DDisplay DDump DHexDump
+		 DDual DGrow );
+@EXPORT_OK = qw( triplevar :tidy );
+push @EXPORT, "DDump_IO";
 
 bootstrap Data::Peek $VERSION;
 
+our $has_perlio;
+our $has_perltidy;
+
+BEGIN {
+    use Config;
+    $has_perlio   = ($Config{useperlio} || "undef") eq "define";
+    $has_perltidy = eval q{use Perl::Tidy; $Perl::Tidy::VERSION};
+    }
+
 ### ############# DDumper () ##################################################
 
 use Data::Dumper;
@@ -45,7 +55,8 @@ my %sk = (
 	    [ reverse sort                           keys %$r ];
 	    },
     );
-my $_sortkeys = 1;
+my  $_sortkeys = 1;
+our $_perltidy = 0;
 
 sub DDsort
 {
@@ -61,6 +72,11 @@ sub import
     foreach my $p (@exp) {
 	exists $sk{$p} and DDsort ($p), next;
 
+	if ($p eq ":tidy") {
+	    $_perltidy = $has_perltidy;
+	    next;
+	    }
+
 	push @etl, $p;
 	}
     __PACKAGE__->export_to_level (1, @etl);
@@ -68,28 +84,47 @@ sub import
 
 sub DDumper
 {
-    local $Data::Dumper::Sortkeys = $_sortkeys;
-    local $Data::Dumper::Indent   = 1;
+    $_perltidy and goto \&DTidy;
+
+    local $Data::Dumper::Sortkeys  = $_sortkeys;
+    local $Data::Dumper::Indent    = 1;
+    local $Data::Dumper::Quotekeys = 0;
+    local $Data::Dumper::Deparse   = 1;
+    local $Data::Dumper::Terse     = 1;
+    local $Data::Dumper::Useqq     = 0;	# I want unicode visible
 
     my $s = Data::Dumper::Dumper @_;
-    $s =~ s!^(\s*)'([^']*)'\s*=>!sprintf "%s%-16s =>", $1, $2!gme;	# Align => '
-    $s =~ s!\bbless\s*\(\s*!bless (!gm and $s =~ s!\s+\)([;,])$!)$1!gm;
-    $s =~ s!^(?= *[]}](?:[;,]|$))!  !gm;
-    $s =~ s!^(\s+)!$1$1!gm;
+    $s =~ s/^(\s*)(.*?)\s*=>/sprintf "%s%-16s =>", $1, $2/gme;  # Align =>
+    $s =~ s/\bbless\s*\(\s*/bless (/gm and $s =~ s/\s+\)([;,])$/)$1/gm;
+    $s =~ s/^(?= *[]}](?:[;,]|$))/  /gm;
+    $s =~ s/^(\s*[{[]) *\n *(?=\S)(?![{[])/$1   /gm;
+    $s =~ s/^(\s+)/$1$1/gm;
 
-    defined wantarray or print STDERR $s;
+    defined wantarray or warn $s;
     return $s;
     } # DDumper
 
+sub DTidy
+{
+    $has_perltidy or goto \&DDumper;
+
+    local $Data::Dumper::Sortkeys  = $_sortkeys;
+    local $Data::Dumper::Indent    = 1;
+    local $Data::Dumper::Quotekeys = 1;
+    local $Data::Dumper::Deparse   = 1;
+    local $Data::Dumper::Terse     = 1;
+    local $Data::Dumper::Useqq     = 0;
+
+    my $s = Data::Dumper::Dumper @_;
+    Perl::Tidy::perltidy (source => \$s, destination => \my $t);
+    $s = $t;
+
+    defined wantarray or warn $s;
+    return $s;
+    } # DTidy
+
 ### ############# DDump () ####################################################
 
-our $has_perlio;
-
-BEGIN {
-    use Config;
-    $has_perlio = ($Config{useperlio} || "undef") eq "define";
-    }
-
 sub _DDump_ref
 {
     my (undef, $down) = (@_, 0);
@@ -158,9 +193,36 @@ sub DDump ($;$)
 
     defined wantarray and return $dump;
 
-    print STDERR $dump;
+    warn $dump;
     } # DDump
 
+sub DHexDump
+{
+    use bytes;
+    my $off = 0;
+    my @out;
+    my $var = @_ ? $_[0] : $_;
+    defined $var or return;
+    my $str = "$var";	# force stringification
+    for (unpack "(A32)*", unpack "H*", $str) {
+	my @b = unpack "(A2)*", $_;
+	my $out = sprintf "%04x ", $off;
+	$out .= " ".($b[$_]||"  ") for 0 ..  7;
+	$out .= " ";
+	$out .= " ".($b[$_]||"  ") for 8 .. 15;
+	$out .= "  ";
+	$out .= ($_ < 0x20 || $_ >= 0x7f ? "." : chr $_) for map { hex $_ } @b;
+	push @out, $out."\n";
+	$off += 16;
+	}
+
+    wantarray and return @out;
+
+    defined wantarray and return join "", @out;
+
+    warn join "", @out;
+    } # DHexDump
+
 "Indent";
 
 __END__
@@ -174,11 +236,13 @@ Data::Peek - A collection of low-level debug facilities
  use Data::Peek;
 
  print DDumper \%hash;    # Same syntax as Data::Dumper
+ DTidy { ref => $ref };
 
  print DPeek \$var;
  my ($pv, $iv, $nv, $rv, $magic) = DDual ($var [, 1]);
  print DPeek for DDual ($!, 1);
  print DDisplay ("ab\nc\x{20ac}\rdef\n");
+ print DHexDump ("ab\nc\x{20ac}\rdef\n");
 
  my $dump = DDump $var;
  my %hash = DDump \@list;
@@ -192,9 +256,12 @@ Data::Peek - A collection of low-level debug facilities
  close $fh;
  print $dump;
 
- use Data::Peek qw( DGrow triplevar );
+ # Imports
+ use Data::Peek qw( :tidy VNR DGrow triplevar );
  my $x = ""; DGrow ($x, 10000);
  my $tv = triplevar ("\N{GREEK SMALL LETTER PI}", 3, "3.1415");
+ DDsort ("R");
+ DDumper [ $x ]; # use of :tidy make DDumper behave as DTidy
 
 =head1 DESCRIPTION
 
@@ -213,21 +280,23 @@ everything set as I like it.
     $Data::Dumper::Sortkeys = 1;
     $Data::Dumper::Indent   = 1;
 
-And the result is further beautified to meet my needs:
+If C<Data::Peek> is C<use>d with import argument C<:tidy>, the result is
+formatted according to L<Perl::Tidy>, see L<DTidy> below, otherwise the
+result is further beautified to meet my needs:
 
   * quotation of hash keys has been removed (with the disadvantage
     that the output might not be parseable again).
   * arrows for hashes are aligned at 16 (longer keys don't align)
   * closing braces and brackets are now correctly aligned
 
-In void context, C<DDumper ()> prints to STDERR.
+In void context, C<DDumper ()> warn ()'s.
 
 Example
 
-  print DDumper { ape => 1, foo => "egg", bar => [ 2, "baz", undef ]};
+  $ perl -MDP \
+    -e'DDumper { ape => 1, foo => "egg", bar => [ 2, "baz", undef ]};'
 
-  $VAR1 = {
-      ape              => 1,
+  {   ape              => 1,
       bar              => [
           2,
           'baz',
@@ -236,6 +305,26 @@ Example
       foo              => 'egg'
       };
 
+=head2 DTidy ($var, ...)
+
+C<DTidy> is an alternative to C<DDumper>, where the output of C<DDumper>
+is formatted using C<Perl::Tidy> (if available) according to your
+C<.perltidyrc> instead of the default behavior, maybe somewhat like (YMMV):
+
+  $ perl -MDP=:tidy \
+    -we'DDumper { ape => 1, foo => "egg", bar => [ 2, "baz", undef ]};'
+  {   'ape' => 1,
+      'bar' => [2, 'baz', undef],
+      'foo' => 'egg'
+      }
+
+If C<Data::Peek> is C<use>d with import argument C<:tidy>, this is the
+default output method for C<DDumper>.
+
+If L<Perl::Tidy> is not available, C<DTidy> will fallback to C<DDumper>.
+
+This idea was shamelessly copied from John McNamara's L<Data::Dumper::Perltidy>.
+
 =head2 DDsort ( 0 | 1 | R | V | VR | VN | VNR )
 
 Set the hash sort algorithm for DDumper. The default is to sort by key value.
@@ -250,16 +339,16 @@ Set the hash sort algorithm for DDumper. The default is to sort by key value.
 
 These can also be passed to import:
 
-  $ perl -MDP=VNR -we'DDumper { foo => 1, bar => 2, zap => 3, gum => 13 }'
-  $VAR1 = {
-      gum              => 13,
+  $ perl -MDP=VNR \
+    -we'DDumper { foo => 1, bar => 2, zap => 3, gum => 13 }'
+  {   gum              => 13,
       zap              => 3,
       bar              => 2,
       foo              => 1
       };
-  $ perl -MDP=V   -we'DDumper { foo => 1, bar => 2, zap => 3, gum => 13 }'
-  $VAR1 = {
-      foo              => 1,
+  $ perl -MDP=V \
+    -we'DDumper { foo => 1, bar => 2, zap => 3, gum => 13 }'
+  {   foo              => 1,
       gum              => 13,
       bar              => 2,
       zap              => 3
@@ -296,6 +385,24 @@ Example
 
   "abc\nde\x{20ac}fg"
 
+=head2 DHexDump
+
+=head2 DHexDump ($var)
+
+Show the (stringified) content of a scalar as a hex-dump.  If C<$var>
+is omitted, C<$_> is dumped. Returns C<undef> or an empty list if
+C<$var> (or C<$_>) is undefined.
+
+In void context, the dump is done to STDERR. In scalar context, the
+complete dump is returned as a single string. In list context, the dump
+is returned as lines.
+
+Example
+
+  print DHexDump "abc\x{0a}de\x{20ac}fg";
+
+  0000  61 62 63 0a 64 65 e2 82  ac 66 67                 abc.de...fg
+
 =head2 my ($pv, $iv, $nv, $rv, $hm) = DDual ($var [, $getmagic])
 
 DDual will return the basic elements in a variable, guaranteeing that no
@@ -322,7 +429,7 @@ In void context, DDual does the equivalent of
       "  RV: ", DPeek ($d[3]), "\n";
     }
   
-=head2 my $LEN = DGrow ($pv, $size)
+=head2 my $len = DGrow ($pv, $size)
 
 Fastest way to preallocate space for a PV scalar. Returns the allocated
 length. If $size is smaller than the already allocated space, it will
@@ -340,7 +447,7 @@ not shrink.
  grow 1622943/s 2512%  967%    --
 
 
-=head2 triplevar ($pv, $iv, $nv)
+=head2 my $tp = triplevar ($pv, $iv, $nv)
 
 When making C<DDual ()> I wondered if it were possible to create triple-val
 scalar variables. L<Scalar::Util> already gives us C<dualvar ()>, that creates
@@ -365,7 +472,7 @@ Example:
 
 A very useful module when debugging is C<Devel::Peek>, but is has one big
 disadvantage: it only prints to STDERR, which is not very handy when your
-code wants to inspect variables al a low level.
+code wants to inspect variables at a low level.
 
 Perl itself has C<sv_dump ()>, which does something similar, but still
 prints to STDERR, and only one level deep.
@@ -394,8 +501,7 @@ Example
   my %h = DDump "abc\x{0a}de\x{20ac}fg";
   print DDumper \%h;
 
-  $VAR1 = {
-      CUR              => '11',
+  {   CUR              => '11',
       FLAGS            => {
           PADBUSY          => 1,
           PADMY            => 1,
@@ -416,8 +522,7 @@ Example
       }, 1;
   print DDumper \%h;
 
-  $VAR1 = {
-      FLAGS            => {
+  {   FLAGS            => {
           PADBUSY          => 1,
           PADMY            => 1,
           ROK              => 1
@@ -470,8 +575,7 @@ Example
 =head2 DDump_IO ($io, $var [, $dig_level])
 
 A wrapper function around perl's internal C<Perl_do_sv_dump ()>, which
-makes C<Devel::Peek> completely superfluous. As PerlIO is only available
-perl version 5.7.3 and up, this function is not available in older perls.
+makes C<Devel::Peek> completely superfluous.
 
 Example
 
@@ -561,8 +665,8 @@ proven to be a big help.
 
 =head1 SEE ALSO
 
-L<Devel::Peek(3)>, L<Data::Dumper(3)>, L<Data::Dump(3)>, L<Devel::Dumpvar>,
-L<Data::Dump::Streamer(3)>
+L<Devel::Peek>, L<Data::Dumper>, L<Data::Dump>, L<Devel::Dumpvar>,
+L<Data::Dump::Streamer>, L<Data::Dumper::Perltidy>, L<Perl::Tidy>.
 
 =head1 AUTHOR
 
@@ -570,7 +674,7 @@ H.Merijn Brand <h.m.brand@xs4all.nl>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2008-2009 H.Merijn Brand
+Copyright (C) 2008-2011 H.Merijn Brand
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
